/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package parameters

import (
	"context"
	"errors"
	"reflect"
	"strings"

	corev1 "k8s.io/api/core/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"

	"k8s.libre.sh/controller-utils/interfaces"
	"sigs.k8s.io/controller-runtime/pkg/client"
)

func Marshal(i interface{}) (*Parameters, error) {
	parameters := &Parameters{}

	t := reflect.TypeOf(i)

	ifv := reflect.ValueOf(i)

	for ii := 0; ii < t.NumField(); ii++ {
		f := t.Field(ii)

		fv := ifv.Field(ii)

		parameter, ok := fv.Interface().(*Parameter)
		if !ok {
			return parameters, errors.New("not parameters type")
		}

		env, ok := f.Tag.Lookup("env")
		if !ok {
			return parameters, errors.New("missing env tag")
		}

		if parameter == nil {
			parameter = &Parameter{}
		}
		if len(parameter.Key) == 0 {
			parameter.Key = env
		}

		*parameters = append(*parameters, parameter)
	}

	return parameters, nil
}

// ByKey returns the parameters sorted by keys
func ByKey(p *Parameters) map[string]*Parameter {
	paramsByKey := map[string]*Parameter{}

	for _, param := range *p {
		paramsByKey[param.Key] = param
	}
	return paramsByKey
}

// KeyPairValues returns the parameters as key:value pairs
func KeyPairValues(p *Parameters) map[string]string {
	data := make(map[string]string)

	for _, param := range *p {
		data[param.Key] = param.Value
	}

	return data
}

// OrderByResourceRef returns a list of parameters ordered reference type and reference name
func OrderByResourceRef(ps *Parameters) map[ParameterType]map[string]*Parameters {

	sorted := make(map[ParameterType]map[string]*Parameters)

	for _, p := range *ps {
		if p != nil {
			// TOFIX only refType ? by default secret ?
			if len(p.ValueFrom.Ref) > 0 {
				// TO DO TO FIX
				if len(p.RefType) == 0 {
					p.RefType = SecretParameter
				}
				if len(sorted[p.RefType]) == 0 {
					sorted[p.RefType] = make(map[string]*Parameters)
				}
				if sorted[p.RefType][p.ValueFrom.Ref] == nil {
					sorted[p.RefType][p.ValueFrom.Ref] = &Parameters{}
				}
				*sorted[p.RefType][p.ValueFrom.Ref] = append(*sorted[p.RefType][p.ValueFrom.Ref], p)
			}
		}

	}

	return sorted
}

// GetDataFromResource return the data as key:value pairs from external resources.
// If resources are owned by the owner object, only generated resources are returned.
// Only secrets and configmaps are supported.
func GetDataFromResource(c client.Client, obj, owner interfaces.Object) (data map[string]string, err error) {

	objData := make(map[string]string)
	data = make(map[string]string)

	objectKey, err := client.ObjectKeyFromObject(obj)
	if err != nil {
		return data, err
	}

	err = c.Get(context.Background(), objectKey, obj)
	if err != nil {
		return data, err
	}

	switch ro := obj.(type) {
	case *corev1.Secret:
		for k, v := range ro.Data {
			objData[k] = string(v)
		}
	case *corev1.ConfigMap:
		objData = ro.Data

	default:
		return data, errors.New("object kind should be ConfigMap or Secret")
	}

	// If object is owner we only want to get the generated data in order to not regenerate it,
	// other parameters are set in crds or valueFrom
	if metav1.IsControlledBy(obj, owner) {
		if len(obj.GetAnnotations()["settings.k8s.libre.sh/generate"]) > 0 {
			for _, param := range strings.Split(obj.GetAnnotations()["settings.k8s.libre.sh/generate"], ",") {
				data[param] = objData[param]
			}
			return data, nil
		}
	} else {
		data = objData
	}

	return data, nil
}
